@@ -1,21 +1,27 @@
+#!/usr/bin/env python
+# coding=utf-8
+
 import hashlib
 import requests
 import six
 import time
+from .exceptions import BinanceAPIException
+from .validation import validate_order
+
 if six.PY2:
     from urllib import urlencode
 elif six.PY3:
     from urllib.parse import urlencode
 
-from .exceptions import BinanceAPIException
-from .validation import validate_order
-
 
 class Client(object):
 
     API_URL = 'https://www.binance.com/api'
+    WEBSITE_URL = 'https://www.binance.com'
     API_VERSION = 'v1'
 
+    _products = None
+
     def __init__(self, api_key, api_secret):
 
         self.API_KEY = api_key
@@ -24,6 +30,7 @@ class Client(object):
 
         # init DNS and SSL cert
         self.ping()
+        self.get_products()
 
     def _init_session(self):
 
@@ -36,6 +43,9 @@ class Client(object):
     def _create_api_uri(self, path):
         return self.API_URL + '/' + self.API_VERSION + '/' + path
 
+    def _create_website_uri(self, path):
+        return self.WEBSITE_URL + '/' + path
+
     def _generate_signature(self, data):
 
         query_string = urlencode(data)
@@ -52,7 +62,7 @@ class Client(object):
             kwargs['data'] = data
         if signed:
             # generate signature
-            kwargs['data']['timestamp'] =int(time.time() * 1000)
+            kwargs['data']['timestamp'] = int(time.time() * 1000)
             kwargs['data']['signature'] = self._generate_signature(kwargs['data'])
 
         if data and method == 'get':
@@ -62,8 +72,23 @@ class Client(object):
         response = getattr(self.session, method)(uri, **kwargs)
         return self._handle_response(response)
 
+    def _request_website(self, method, path, **kwargs):
+
+        uri = self._create_website_uri(path)
+
+        data = kwargs.get('data', None)
+        if data and isinstance(data, dict):
+            kwargs['data'] = data
+
+        if data and method == 'get':
+            kwargs['params'] = kwargs['data']
+            del(kwargs['data'])
+
+        response = getattr(self.session, method)(uri, **kwargs)
+        return self._handle_response(response)
+
     def _handle_response(self, response):
-        """Internal helper for handling API responses from the Coinbase server.
+        """Internal helper for handling API responses from the Binance server.
         Raises the appropriate exceptions when necessary; otherwise, returns the
         response.
         """
@@ -83,6 +108,29 @@ class Client(object):
     def _delete(self, path, signed=False, **kwargs):
         return self._request('delete', path, signed, **kwargs)
 
+    def _parse_products(self, products):
+        """
+        Parse the response from get_products to use for validation
+        :param products:
+        :return:
+        """
+        self._products = {}
+        if 'data' in products:
+            products = products['data']
+        for p in products:
+            self._products[p['symbol']] = p
+
+    # Website Endpoints
+
+    def get_products(self):
+        """
+        Return list of products currently listed on Binance
+        :return:
+        """
+        products = self._request_website('get', 'exchange/public/product')
+        self._parse_products(products)
+        return products
+
     # General Endpoints
 
     def ping(self):
@@ -103,6 +151,20 @@ class Client(object):
 
     # Market Data Endpoints
 
+    def get_all_tickers(self):
+        """
+        Get last price for all markets
+        :return:
+        """
+        return self._get('ticker/allPrices')
+
+    def get_orderbook_tickers(self):
+        """
+        Get first bid and ask entry in the order book for all markets
+        :return:
+        """
+        return self._get('ticker/allBookTickers')
+
     def get_order_book(self, **params):
         """
         Get the Order Book for the market
@@ -155,43 +217,47 @@ class Client(object):
 
     # Account Endpoints
 
-    def create_order(self, **params):
+    def create_order(self, disable_validation=False, **params):
         """
         Send in a new order
         https://www.binance.com/restapipub.html#new-order--signed
+        :param disable_validation: disable client side order validation
         :param params:
             symbol - required
             side - required
             type - required
-            timeInForce - required
+            timeInForce - required if limit order
             quantity - required
-            price - required
+            price - required if limit order
             newClientOrderId - A unique id for the order. Automatically generated if not sent.
             stopPrice - Used with stop orders
             icebergQty - Used with iceberg orders
         :return:
         """
-        validate_order(params)
+        if not disable_validation:
+            validate_order(params, self._products)
         return self._post('order', True, data=params)
 
-    def create_test_order(self, **params):
+    def create_test_order(self, disable_validation=False, **params):
         """
         Test new order creation and signature/recvWindow long. Creates and validates a new order but does not send it into the matching engine.
         https://www.binance.com/restapipub.html#test-new-order-signed
+        :param disable_validation: disable client side order validation
         :param params:
             symbol - required
             side - required enum
             type - required enum
-            timeInForce - required enum
+            timeInForce - required if limit order
             quantity - required
-            price - required
+            price - required if limit order
             newClientOrderId - A unique id for the order. Automatically generated if not sent.
             stopPrice - Used with stop orders
             icebergQty - Used with iceberg orders
             recvWindow - the number of milliseconds the request is valid for
         :return:
         """
-        validate_order(params)
+        if not disable_validation:
+            validate_order(params, self._products)
         return self._post('order/test', True, data=params)
 
     def get_order(self, **params):
@@ -249,7 +315,7 @@ class Client(object):
     # User Stream Endpoints
     def get_account(self, **params):
         """
-        Get all account orders; active, canceled, or filled.
+        Get current account information.
         https://www.binance.com/restapipub.html#account-information-signed
         :param params:
             recvWindow - the number of milliseconds the request is valid for
@@ -273,11 +339,26 @@ class Client(object):
     # User Stream Endpoints
 
     def stream_get_listen_key(self):
+        """
+        Start a new user data stream and return the listen key
+        https://www.binance.com/restapipub.html#start-user-data-stream-api-key
+        :return:
+        """
         res = self._post('userDataStream', False, data={})
         return res['listenKey']
 
     def stream_keepalive(self, **params):
+        """
+        PING a user data stream to prevent a time out.
+        https://www.binance.com/restapipub.html#keepalive-user-data-stream-api-key
+        :return:
+        """
         return self._put('userDataStream', False, data=params)
 
     def stream_close(self, **params):
-        return self._delete('userDataStream', False, data=params)
\ No newline at end of file
+        """
+        Close out a user data stream.
+        https://www.binance.com/restapipub.html#close-user-data-stream-api-key
+        :return:
+        """
+        return self._delete('userDataStream', False, data=params)
